"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateImports = void 0;

const generateImports = _ref => {
  let {
    schemaName,
    apiDirectory,
    queryClientDir,
    schemaImports,
    queryImports
  } = _ref;
  const importTypes = schemaImports.join(',');
  let imports = [];

  if (queryImports.includes('query')) {
    imports = [...imports, 'useQuery', 'UseQueryOptions', 'QueryKey', 'SetDataOptions', 'QueryFilters'];
  }

  if (queryImports.includes('infiniteQuery')) {
    imports = [...imports, 'useInfiniteQuery', 'UseInfiniteQueryOptions', 'QueryKey'];
  }

  if (queryImports.includes('mutation')) {
    imports = [...imports, 'UseMutationOptions', 'useMutation'];
  }

  const importString = [...new Set(imports)].join(',');
  return `
  import {
    ${importString}
  } from '@tanstack/react-query';
  
  import { AxiosError } from 'axios';
  import { api } from '${apiDirectory}';
  import { queryClient } from '${queryClientDir}';

  import {${importTypes}} from './${schemaName}'

  type Updater<TInput, TOutput> = TOutput | ((input: TInput) => TOutput);  
  `;
};

exports.generateImports = generateImports;
//# sourceMappingURL=generateImports.js.map