"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateSchemas = void 0;

var _case = _interopRequireDefault(require("case"));

var _lodash = _interopRequireDefault(require("lodash"));

var _utils = require("./utils.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  isEmpty
} = _lodash.default;
const {
  pascal
} = _case.default;

/**
 * Generate the interface string
 */
const generateInterface = (name, schema) => {
  const scalar = (0, _utils.getScalar)(schema);
  return `${(0, _utils.formatDescription)(schema.description)}
       export type ${pascal(name)} = ${scalar}
     `;
};
/**
 * Extract all types from #/components/schemas
 */


const generateSchemasDefinition = function () {
  let schemas = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (isEmpty(schemas)) {
    return '';
  }

  return '\n // SCEHMAS \n' + Object.entries(schemas).map(_ref => {
    let [name, schema] = _ref;
    return !(0, _utils.isReference)(schema) && (!schema.type || schema.type === 'object') && !schema.allOf && !schema.oneOf && !(0, _utils.isReference)(schema) && !schema.nullable ? generateInterface(name, schema) : `${(0, _utils.formatDescription)((0, _utils.isReference)(schema) ? undefined : schema.description)} export type ${pascal(name)} = ${(0, _utils.resolveValue)(schema)};`;
  }).join('\n\n') + '\n';
};
/**
 * Extract all types from #/components/requestBodies
 */


const generateRequestBodiesDefinition = function () {
  let requestBodies = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (isEmpty(requestBodies)) {
    return '';
  }

  return '\n // REQUEST BODIES \n' + Object.entries(requestBodies).map(_ref2 => {
    let [name, requestBody] = _ref2;
    const doc = (0, _utils.getDocs)(requestBody);
    const type = (0, _utils.getResReqTypes)([['', requestBody]]);
    const isEmptyInterface = type === '{}';

    if (isEmptyInterface) {
      return `${doc}\nexport type ${pascal(name)}RequestBody = ${type}`;
    } else if (type.includes('{') && !type.includes('|') && !type.includes('&')) {
      return `${doc}\nexport type ${pascal(name)}RequestBody = ${type}`;
    } else {
      return `${doc}\nexport type ${pascal(name)}RequestBody = ${type};`;
    }
  }).join('\n\n') + '\n';
};
/**
 * Extract all types from #/components/responses
 */


const generateResponsesDefinition = function () {
  let responses = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (isEmpty(responses)) {
    return '';
  }

  return '\n // RESPONSES \n' + Object.entries(responses).map(_ref3 => {
    let [name, response] = _ref3;
    const doc = (0, _utils.getDocs)(response);
    const type = (0, _utils.getResReqTypes)([['', response]]);
    const isEmptyInterface = type === '{}';

    if (isEmptyInterface) {
      return `export type RQ${pascal(name)}Response = ${type}`;
    } else if (type.includes('{') && !type.includes('|') && !type.includes('&')) {
      return `${doc}export type RQ${pascal(name)}Response = ${type}`;
    } else {
      return `${doc}export type RQ${pascal(name)}Response = ${type};`;
    }
  }).join('\n\n') + '\n';
};
/**
 * Extract all types from #/components/schemas
 */


const generateSchemas = _ref4 => {
  var _spec$components, _spec$components2, _spec$components3;

  let {
    spec
  } = _ref4;
  const schemaOutput = generateRequestBodiesDefinition((_spec$components = spec.components) === null || _spec$components === void 0 ? void 0 : _spec$components.requestBodies) + generateResponsesDefinition((_spec$components2 = spec.components) === null || _spec$components2 === void 0 ? void 0 : _spec$components2.responses) + generateSchemasDefinition((_spec$components3 = spec.components) === null || _spec$components3 === void 0 ? void 0 : _spec$components3.schemas);
  return schemaOutput;
};

exports.generateSchemas = generateSchemas;
//# sourceMappingURL=generateSchemas.js.map