"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.importSpecs = importSpecs;

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = require("fs");

var _path = require("path");

var _convertSwaggerFile = require("./convertSwaggerFile.js");

var _generateHooks = require("./generateHooks.js");

var _generateImports = require("./generateImports.js");

var _generateSchemas = require("./generateSchemas.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function importSpecs(_ref) {
  let {
    sourceDirectory,
    exportDirectory,
    apiDirectory,
    queryClientDir,
    headerFilters,
    overrides
  } = _ref;
  (0, _fs.readdir)(sourceDirectory, function (err, filenames) {
    if (err) {
      console.log(err);
      throw err;
    }

    filenames.map(async filename => {
      try {
        const data = (0, _fs.readFileSync)((0, _path.join)(process.cwd(), sourceDirectory + '/' + filename), 'utf-8');
        const {
          ext
        } = (0, _path.parse)(sourceDirectory + '/' + filename);
        const format = ['.yaml', '.yml'].includes(ext.toLowerCase()) ? 'yaml' : 'json';
        let spec = await (0, _convertSwaggerFile.convertSwaggerFile)(data, format);
        const schemaName = `useQueries${filename.split('.')[0]}.schema`;
        const hooksName = `useQueries${filename.split('.')[0]}`;
        const name = filename.split('.')[0];
        (0, _fs.mkdirSync)((0, _path.join)(process.cwd(), `${exportDirectory}/${name}`), {
          recursive: true
        });
        const operationIds = [];
        let hooks = '';
        let schemaImportsArray = [];
        let collectedQueryImports = [];
        Object.entries(spec.paths).forEach(_ref2 => {
          let [route, verbs] = _ref2;
          Object.entries(verbs).forEach(_ref3 => {
            let [verb, operation] = _ref3;

            if (['get', 'post', 'patch', 'put', 'delete'].includes(verb) && !operation.deprecated) {
              const {
                implementation,
                imports,
                queryImports
              } = (0, _generateHooks.createHook)({
                operation,
                verb,
                route: (spec.basePath || '') + route,
                operationIds,
                parameters: verbs.parameters,
                schemasComponents: spec.components,
                headerFilters,
                overrides
              });
              hooks += implementation;
              imports.forEach(element => {
                const formattedImport = element.replace('[]', '');

                if (!schemaImportsArray.includes(formattedImport) && element !== 'void' && element !== 'string' && !element.includes('{')) {
                  schemaImportsArray.push(formattedImport);
                }
              });
              queryImports.forEach(element => {
                if (!schemaImportsArray.includes(element)) {
                  collectedQueryImports.push(element);
                }
              });
            }
          });
        });
        const imports = (0, _generateImports.generateImports)({
          apiDirectory,
          queryClientDir,
          schemaName,
          schemaImports: schemaImportsArray,
          queryImports: collectedQueryImports
        });
        (0, _fs.writeFileSync)((0, _path.join)(process.cwd(), `${exportDirectory}/${name}/${hooksName}.tsx`), imports + hooks);
        const schemas = (0, _generateSchemas.generateSchemas)({
          spec
        });
        (0, _fs.writeFileSync)((0, _path.join)(process.cwd(), `${exportDirectory}/${name}/${schemaName}.tsx`), schemas);
        console.log(_chalk.default.green(`🎉 [${filename}] Your OpenAPI spec has been converted into react query hooks`));
      } catch (error) {
        if (error.code === 'EISDIR') {
          console.log(_chalk.default.red('nested folder structure not supported'));
          return;
        }

        console.log(_chalk.default.red(`[${filename}]:`), _chalk.default.red(error));
      }
    });
  });
}
//# sourceMappingURL=importSpecs.js.map