"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
exports.__esModule = true;
exports.createHook = void 0;
var lodash_1 = require("lodash");
var get = lodash_1["default"].get, groupBy = lodash_1["default"].groupBy;
var utils_js_1 = require("./utils.js");
var case_1 = require("case");
var chalk_1 = require("chalk");
var pascal = case_1["default"].pascal, camel = case_1["default"].camel;
var IdentifierRegexp = /^[a-zA-Z_$][a-zA-Z0-9_$]*$/;
/**
 * Return every params in a path
 */
var getParamsInPath = function (path) {
    var n;
    var output = [];
    var templatePathRegex = /\{(\w+)}/g;
    while ((n = templatePathRegex.exec(path)) !== null) {
        output.push(n[1]);
    }
    return output;
};
/**
 * Generate a react-query component from openapi operation specs
 */
var createHook = function (_a) {
    var operation = _a.operation, verb = _a.verb, route = _a.route, operationIds = _a.operationIds, parameters = _a.parameters, schemasComponents = _a.schemasComponents, headerFilters = _a.headerFilters, overrides = _a.overrides;
    var _b = operation.operationId, operationId = _b === void 0 ? route.replace('/', '') : _b;
    if (operationId === '*') {
        throw new Error("Invalid operationId/Route set for ".concat(verb, " ").concat(route));
    }
    if (operationIds.includes(operationId)) {
        return { implementation: '', imports: [], queryImports: [] };
    }
    operationIds.push(operationId);
    route = route.replace(/\{/g, '${').replace('//', '/'); // `/pet/{id}` => `/pet/${id}`
    // Remove the last param of the route if we are in the DELETE case
    var lastParamInTheRoute = null;
    var componentName = pascal(operationId);
    var isOk = function (_a) {
        var statusCode = _a[0];
        return statusCode.toString().startsWith('2');
    };
    var responseTypes = (0, utils_js_1.getResReqTypes)(Object.entries(operation.responses).filter(isOk)) || 'void';
    var requestBodyTypes = (0, utils_js_1.getResReqTypes)([['body', operation.requestBody]]);
    var imports = [responseTypes];
    var queryImports = [];
    var paramsInPath = getParamsInPath(route).filter(function (param) { return !(verb === 'delete' && param === lastParamInTheRoute); });
    var _c = groupBy(__spreadArray(__spreadArray([], (parameters || []), true), (operation.parameters || []), true).map(function (p) {
        if ((0, utils_js_1.isReference)(p)) {
            return get(schemasComponents, p.$ref.replace('#/components/', '').replace('/', '.'));
        }
        else {
            return p;
        }
    }), 'in'), _d = _c.query, queryParams = _d === void 0 ? [] : _d, _e = _c.path, pathParams = _e === void 0 ? [] : _e, _f = _c.header, header = _f === void 0 ? [] : _f;
    var headerParams = header.filter(function (p) { return !(headerFilters === null || headerFilters === void 0 ? void 0 : headerFilters.includes(p.name)); });
    var enabled = [];
    // TODO: extract all requestBody or remove useQuery variants
    var enabledParam = '!!params';
    __spreadArray(__spreadArray(__spreadArray([], queryParams, true), pathParams, true), headerParams, true).forEach(function (item) {
        if (item.required) {
            enabled.push("[\"".concat(item.name, "\"]"));
            if (enabledParam && enabledParam !== '!!params') {
                enabledParam += "&& params['".concat(item.name, "'] != null");
            }
            else {
                enabledParam = "params['".concat(item.name, "'] != null");
            }
        }
    });
    // `!props${enabled.join('== null && !props')}`;
    var paramsTypes = paramsInPath
        .map(function (p) {
        try {
            var _a = pathParams.find(function (i) { return i.name === p; }), name_1 = _a.name, required = _a.required, schema = _a.schema;
            return "".concat(name_1).concat(required ? '' : '?', ": ").concat((0, utils_js_1.resolveValue)(schema));
        }
        catch (err) {
            throw new Error("The path params ".concat(p, " can't be found in parameters (").concat(operationId, ")"));
        }
    })
        .join('; ');
    var queryParamsType = queryParams
        .map(function (p) {
        var processedName = IdentifierRegexp.test(p.name) ? p.name : "\"".concat(p.name, "\"");
        return "".concat((0, utils_js_1.formatDescription)(p.description), "\n      ").concat(processedName).concat(p.required ? '' : '?', ": ").concat((0, utils_js_1.resolveValue)(p.schema));
    })
        .join(';\n  ');
    var headerType = headerParams
        .map(function (p) {
        try {
            var _a = headerParams.find(function (i) { return i.name === p.name; }), name_2 = _a.name, required = _a.required, schema = _a.schema;
            return "\"".concat(name_2, "\"").concat(required ? '' : '?', ": ").concat((0, utils_js_1.resolveValue)(schema));
        }
        catch (err) {
            throw new Error("The path params ".concat(p, " can't be found in parameters (").concat(operationId, ")"));
        }
    })
        .join('; ');
    // Retrieve the type of the param for delete verb
    var lastParamInTheRouteDefinition = operation.parameters && lastParamInTheRoute
        ? operation.parameters.find(function (p) {
            if ((0, utils_js_1.isReference)(p)) {
                return false;
            }
            return p.name === lastParamInTheRoute;
        }) // Reference is not possible
        : { schema: { type: 'string' } };
    if (!lastParamInTheRouteDefinition) {
        throw new Error("The path params ".concat(lastParamInTheRoute, " can't be found in parameters (").concat(operationId, ")"));
    }
    var defaultDescription = "type: ".concat(verb, "\noperationId: ").concat(operationId, "\nurl: ").concat(route);
    var description = (0, utils_js_1.formatDescription)(operation.summary && operation.description
        ? "".concat(defaultDescription, "\n\n").concat(operation.summary, "\n\n").concat(operation.description)
        : "".concat(defaultDescription));
    var output = "\n\n".concat(description);
    var headerParam = headerType && headerType !== 'void' ? "".concat(headerType, ";") : '';
    var queryParam = queryParamsType && queryParamsType !== 'void' ? "".concat(queryParamsType) : '';
    var requestBodyComponent = requestBodyTypes && requestBodyTypes !== 'void' ? "".concat(requestBodyTypes) : '';
    if (requestBodyComponent) {
        imports.push(requestBodyComponent);
    }
    var isUpdateRequest = ['post', 'patch', 'put'].includes(verb);
    var generateProps = function (props) {
        return props.map(function (item) { return "[\"".concat(item.name, "\"]: props[\"").concat(item.name, "\"]"); }).join(',');
    };
    var generateBodyProps = function () {
        var _a;
        var definitionKey = Object.keys((schemasComponents === null || schemasComponents === void 0 ? void 0 : schemasComponents.schemas) || {}).find(function (key) { return pascal(key) === requestBodyComponent; });
        if (definitionKey) {
            var scheme = (_a = schemasComponents === null || schemasComponents === void 0 ? void 0 : schemasComponents.schemas) === null || _a === void 0 ? void 0 : _a[definitionKey];
            return Object.keys(scheme.properties)
                .map(function (item) { return "[\"".concat(item, "\"]: props[\"").concat(item, "\"]"); })
                .join(',');
        }
    };
    var fetchName = camel(componentName);
    var createQueryHooks = function (emptyParams) {
        var params = emptyParams ? '' : "params: ".concat(componentName, "Params,");
        var key = emptyParams ? '' : "params";
        var mutationParams = emptyParams ? 'void' : "".concat(componentName, "Params");
        var queryParamType = emptyParams ? '' : "".concat(componentName, "Params &");
        var filterParams = emptyParams
            ? '{ filters }: { filters?: QueryFilters }'
            : "{ params, filters }: { params: ".concat(componentName, "Params, filters?: QueryFilters }");
        var cacheParams = emptyParams
            ? "{updater, options}: {updater: Updater<".concat(responseTypes, " | undefined, ").concat(responseTypes, " | undefined>, options?: SetDataOptions | undefined}")
            : "{params, updater, options}: {params: ".concat(componentName, "Params, updater: Updater<").concat(responseTypes, " | undefined, ").concat(responseTypes, " | undefined>, options?: SetDataOptions | undefined}");
        var queryKey = emptyParams
            ? "use".concat(componentName, "Query.baseKey()")
            : "[...use".concat(componentName, "Query.baseKey(), params]");
        var createQuery = function () { return "\n    type ".concat(componentName, "QueryProps<T = ").concat(responseTypes, "> = ").concat(queryParamType, " {\n      options?: UseQueryOptions<").concat(responseTypes, ", AxiosError, T, any> \n    }\n    export function use").concat(componentName, "Query<T = ").concat(responseTypes, ">({ options = {}, ...params }: ").concat(componentName, "QueryProps<T>) { \n      return useQuery(use").concat(componentName, "Query.queryKey(").concat(key, "), async () => ").concat(fetchName, "(").concat(key, "), { enabled: ").concat(enabledParam, ", ...options });\n    }\n\n    use").concat(componentName, "Query.baseKey = (): QueryKey => [\"").concat(componentName.toLowerCase(), "\"];\n  \n    use").concat(componentName, "Query.queryKey = (").concat(params, "): QueryKey => ").concat(queryKey, ";\n  \n    use").concat(componentName, "Query.updateCache = (").concat(cacheParams, ") => queryClient.setQueryData<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "), updater, options);\n    \n    use").concat(componentName, "Query.getQueryState = (").concat(filterParams, ")=> queryClient.getQueryState<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "), filters);\n    \n    use").concat(componentName, "Query.getQueryData = (").concat(filterParams, ")=> queryClient.getQueryData<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "), filters);\n    \n    use").concat(componentName, "Query.prefetch = (").concat(params, ") => queryClient.prefetchQuery<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "), ()=> ").concat(fetchName, "(").concat(key, "));\n  \n    use").concat(componentName, "Query.cancelQueries = (").concat(params, ") => queryClient.cancelQueries(use").concat(componentName, "Query.queryKey(").concat(key, "))\n  \n    use").concat(componentName, "Query.invalidate = (").concat(params, ") => queryClient.invalidateQueries<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "));\n  \n    use").concat(componentName, "Query.refetchStale = (").concat(params, ") => queryClient.refetchQueries<").concat(responseTypes, ">(use").concat(componentName, "Query.queryKey(").concat(key, "), { stale: true });\n  "); };
        var getInfiniteQuery = function (_a) {
            var pageParam = _a.pageParam;
            return "\n    type ".concat(componentName, "QueryProps<T = ").concat(responseTypes, "> = ").concat(queryParamType, " {\n      options: UseInfiniteQueryOptions<").concat(responseTypes, ", AxiosError, T, any> \n    }\n    export function use").concat(componentName, "Query<T = ").concat(responseTypes, ">({ options = {}, ...params }: ").concat(componentName, "QueryProps<T>) { \n      return useInfiniteQuery(use").concat(componentName, "Query.queryKey(").concat(key, "), async ({ pageParam = 0 }) => ").concat(fetchName, "({").concat(pageParam, ": pageParam, ...params}), { enabled: ").concat(enabledParam, ", ...options });\n    }\n\n    use").concat(componentName, "Query.baseKey = (): QueryKey => [\"").concat(componentName.toLowerCase(), "\"];\n  \n    use").concat(componentName, "Query.queryKey = (").concat(params, "): QueryKey => ").concat(queryKey, ";\n  \n  ");
        };
        var createMutation = function () { return "\n    type ".concat(componentName, "MutationProps<T> = {\n      options?: UseMutationOptions<").concat(responseTypes, ", AxiosError, ").concat(mutationParams, ", T> \n    }\n    export function use").concat(componentName, "Mutation<T = ").concat(responseTypes, ">(props?: ").concat(componentName, "MutationProps<T>) { \n      return useMutation(async (").concat(key, ") => ").concat(fetchName, "(").concat(key, "), props?.options)\n    };\n    "); };
        var override = overrides === null || overrides === void 0 ? void 0 : overrides[operationId];
        if ((override === null || override === void 0 ? void 0 : override.type) === 'query') {
            console.log(chalk_1["default"].blue("\u2699\uFE0F  [".concat(operationId, "] has been changed to a useQuery hook")));
            queryImports.push('query');
            return createQuery();
        }
        if ((override === null || override === void 0 ? void 0 : override.type) === 'mutation') {
            queryImports.push('mutation');
            console.log(chalk_1["default"].blue("\u2699\uFE0F  [".concat(operationId, "] has been changed to a useMutation hook")));
            return createMutation();
        }
        if ((override === null || override === void 0 ? void 0 : override.type) === 'infiniteQuery') {
            console.log(chalk_1["default"].blue("\u2699\uFE0F  [".concat(operationId, "] has been changed to a useInfiniteQuery hook")));
            queryImports.push('infiniteQuery');
            return getInfiniteQuery({ pageParam: override.infiniteQueryParm });
        }
        if (verb === 'get') {
            queryImports.push('query');
            return createQuery();
        }
        queryImports.push('mutation');
        return createMutation();
    };
    output += createQueryHooks(!requestBodyComponent && !paramsInPath.length && !queryParam && !headerParam);
    if (!requestBodyComponent && !paramsInPath.length && !queryParam && !headerParam) {
        output += "\n    const ".concat(fetchName, " = async () => {\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`);\n      return result.data;\n    }\n    ");
    }
    if (!requestBodyComponent && paramsInPath.length && queryParam && !headerParam) {
        var config = isUpdateRequest ? 'body,{params}' : '{params}';
        output += "\n    type ".concat(componentName, "Params = {\n      ").concat(paramsTypes, "\n      ").concat(queryParamsType, "; \n    }\n\n     const ").concat(fetchName, " = async (props:").concat(componentName, "Params) => {\n      const {").concat(paramsInPath.join(', '), ", ...params} = props\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, ").concat(config, ")\n      return result.data;\n    }");
    }
    if (!requestBodyComponent && paramsInPath.length && !queryParam && !headerParam) {
        output += "\n    type ".concat(componentName, "Params = {\n      ").concat(paramsTypes, "\n    }\n\n    const ").concat(fetchName, " = async (props: ").concat(componentName, "Params ) => {\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route.replace(/\{/g, '{props.'), "`);\n      return result.data;\n    }\n    ");
    }
    if (!requestBodyComponent && !paramsInPath.length && queryParam && !headerParam) {
        var config = isUpdateRequest ? 'null,{params}' : '{params}';
        output += "\n    type ".concat(componentName, "Params = { \n      ").concat(queryParamsType, " \n    }\n\n    const ").concat(fetchName, " = async (params: ").concat(componentName, "Params) => {\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, ").concat(config, ")\n      return result.data;\n    }\n    ");
    }
    if (!requestBodyComponent && !paramsInPath.length && queryParam && headerParam) {
        var config = isUpdateRequest ? 'null,{headers, params: queryParams}' : '{headers, params: queryParams}';
        output += "\n      type ".concat(componentName, "Params = {\n        ").concat(headerParam, "\n        ").concat(queryParamsType, "\n      }\n      const ").concat(fetchName, " = async (props: ").concat(componentName, "Params) => {\n        const headers = {").concat(generateProps(header), "}\n        const queryParams = {").concat(generateProps(queryParams), "}\n        const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, ").concat(config, ")\n        return result.data\n      }");
    }
    if (!requestBodyComponent && !paramsInPath.length && !queryParam && headerParam) {
        var config = isUpdateRequest ? 'null,{headers}' : '{headers}';
        output += "\n      type ".concat(componentName, "Params = {\n        ").concat(headerParam, "\n      };\n  \n      const ").concat(fetchName, " = async (headers: ").concat(componentName, "Params) => {\n        const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, ").concat(config, ");\n        return result.data;\n      }\n      ");
    }
    if (requestBodyComponent && !paramsInPath.length && !queryParam && !headerParam) {
        output += "\n    type ".concat(componentName, "Params = ").concat(requestBodyComponent, "\n\n    const ").concat(fetchName, " = async (body: ").concat(componentName, "Params) => {\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, body)\n      return result.data\n    } \n      ");
    }
    if (requestBodyComponent && !paramsInPath.length && queryParam && !headerParam) {
        var config = isUpdateRequest ? 'null,{headers, params: queryParams}' : '{headers, params: queryParams}';
        var queryParamsProps = queryParams.map(function (item) { return "[\"".concat(item.name, "\"]: props[\"").concat(item.name, "\"]"); });
        output += "\n    type ".concat(componentName, "Params = ").concat(requestBodyComponent, "  & {\n      ").concat(queryParamsType, "\n    }\n\n    type ").concat(componentName, "QueryProps<T = ").concat(responseTypes, "> = ").concat(componentName, "Params & {\n      const body =  {").concat(generateBodyProps(), "}\n      const params =  {").concat(generateProps(queryParams), "}\n      options?: UseQueryOptions<").concat(responseTypes, ", AxiosError, T, any> \n    }\n\n     const ").concat(fetchName, " = async ({body, params}: ").concat(componentName, "Params) => {   \n      const queryParams = {").concat(queryParamsProps.join(','), "}\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, ").concat(config, ")\n      return result.data\n    }\n    ");
    }
    if (requestBodyComponent && !paramsInPath.length && !queryParam && headerParam) {
        output += "\n    type ".concat(componentName, "Params =  ").concat(requestBodyComponent, " & {\n      ").concat(headerParam, "\n    };\n\n    const ").concat(fetchName, " = async (props: ").concat(componentName, "Params) => {\n      const headers = {").concat(generateProps(header), "}\n      const body =  {").concat(generateBodyProps(), "}\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, body, {headers})\n      return result.data\n    }\n    ");
    }
    if (requestBodyComponent && !paramsInPath.length && queryParam && headerParam) {
        output += "\n    type ".concat(componentName, "Params = ").concat(requestBodyComponent, " & {\n      ").concat(headerParam, "\n      ").concat(paramsTypes, "\n    };\n\n    const ").concat(fetchName, " = async (props: ").concat(componentName, "Params) => {\n      const body =  {").concat(generateBodyProps(), "}\n      const headers = {").concat(generateProps(header), "}\n      const params =  {").concat(generateProps(queryParams), "}\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route, "`, body, {headers, params})\n      return result.data\n    }\n    ");
    }
    if (requestBodyComponent && paramsInPath.length && !queryParam && !headerParam) {
        output += "\n    type ".concat(componentName, "Params = ").concat(requestBodyComponent, " & {\n      ").concat(headerParam, "\n      ").concat(paramsTypes, "\n    };\n\n    const ").concat(fetchName, " = async (props: ").concat(componentName, "Params) => {\n      const body =  {").concat(generateBodyProps(), "}\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route.replace(/\{/g, '{props.'), "`, body)\n      return result.data\n    }\n    ");
    }
    if (requestBodyComponent && paramsInPath.length && queryParam && !headerParam) {
        output += "// TODO: NOT SUPPORTED requestBodyComponent && paramsInPath && queryParam)";
    }
    if (requestBodyComponent && paramsInPath.length && queryParam && headerParam) {
        output += "// TODO: NOT SUPPORTED requestBodyComponent && paramsInPath && queryParam && headerParam)";
    }
    if (!requestBodyComponent && paramsInPath.length && !queryParam && headerParam) {
        var config = isUpdateRequest ? 'null,{headers}' : '{headers}';
        output += "\n    type ".concat(componentName, "Params = {\n      ").concat(headerParam, "\n      ").concat(paramsTypes, "\n    };\n\n    const ").concat(fetchName, " = async (props: ").concat(componentName, "Params) => {\n      const headers = {").concat(generateProps(header), "}\n      const result = await api.").concat(verb, "<").concat(responseTypes, ">(`").concat(route.replace(/\{/g, '{props.'), "`, ").concat(config, ")\n      return result.data\n    }\n    ");
    }
    if (!requestBodyComponent && paramsInPath.length && queryParam && headerParam) {
        output += "// TODO: NOT SUPPORTED (paramsInPath && queryParam && headerParam)";
    }
    if (requestBodyComponent && paramsInPath.length && !queryParam && headerParam) {
        output += "// TODO: NOT SUPPORTED (requestBodyComponent && paramsInPath && headerParam)";
    }
    return { implementation: output, imports: imports, queryImports: queryImports };
};
exports.createHook = createHook;
