"use strict";
exports.__esModule = true;
exports.generateSchemas = void 0;
var case_1 = require("case");
var lodash_1 = require("lodash");
var isEmpty = lodash_1["default"].isEmpty;
var pascal = case_1["default"].pascal;
var utils_js_1 = require("./utils.js");
var utils_js_2 = require("./utils.js");
/**
 * Generate the interface string
 */
var generateInterface = function (name, schema) {
    var scalar = (0, utils_js_1.getScalar)(schema);
    return "".concat((0, utils_js_1.formatDescription)(schema.description), "\n       export type ").concat(pascal(name), " = ").concat(scalar, "\n     ");
};
/**
 * Extract all types from #/components/schemas
 */
var generateSchemasDefinition = function (schemas) {
    if (schemas === void 0) { schemas = {}; }
    if (isEmpty(schemas)) {
        return '';
    }
    return ('\n // SCEHMAS \n' +
        Object.entries(schemas)
            .map(function (_a) {
            var name = _a[0], schema = _a[1];
            return !(0, utils_js_1.isReference)(schema) &&
                (!schema.type || schema.type === 'object') &&
                !schema.allOf &&
                !schema.oneOf &&
                !(0, utils_js_1.isReference)(schema) &&
                !schema.nullable
                ? generateInterface(name, schema)
                : "".concat((0, utils_js_1.formatDescription)((0, utils_js_1.isReference)(schema) ? undefined : schema.description), " export type ").concat(pascal(name), " = ").concat((0, utils_js_1.resolveValue)(schema), ";");
        })
            .join('\n\n') +
        '\n');
};
/**
 * Extract all types from #/components/requestBodies
 */
var generateRequestBodiesDefinition = function (requestBodies) {
    if (requestBodies === void 0) { requestBodies = {}; }
    if (isEmpty(requestBodies)) {
        return '';
    }
    return ('\n // REQUEST BODIES \n' +
        Object.entries(requestBodies)
            .map(function (_a) {
            var name = _a[0], requestBody = _a[1];
            var doc = (0, utils_js_2.getDocs)(requestBody);
            var type = (0, utils_js_2.getResReqTypes)([['', requestBody]]);
            var isEmptyInterface = type === '{}';
            if (isEmptyInterface) {
                return "".concat(doc, "\nexport type ").concat(pascal(name), "RequestBody = ").concat(type);
            }
            else if (type.includes('{') && !type.includes('|') && !type.includes('&')) {
                return "".concat(doc, "\nexport type ").concat(pascal(name), "RequestBody = ").concat(type);
            }
            else {
                return "".concat(doc, "\nexport type ").concat(pascal(name), "RequestBody = ").concat(type, ";");
            }
        })
            .join('\n\n') +
        '\n');
};
/**
 * Extract all types from #/components/responses
 */
var generateResponsesDefinition = function (responses) {
    if (responses === void 0) { responses = {}; }
    if (isEmpty(responses)) {
        return '';
    }
    return ('\n // RESPONSES \n' +
        Object.entries(responses)
            .map(function (_a) {
            var name = _a[0], response = _a[1];
            var doc = (0, utils_js_2.getDocs)(response);
            var type = (0, utils_js_2.getResReqTypes)([['', response]]);
            var isEmptyInterface = type === '{}';
            if (isEmptyInterface) {
                return "export type RQ".concat(pascal(name), "Response = ").concat(type);
            }
            else if (type.includes('{') && !type.includes('|') && !type.includes('&')) {
                return "".concat(doc, "export type RQ").concat(pascal(name), "Response = ").concat(type);
            }
            else {
                return "".concat(doc, "export type RQ").concat(pascal(name), "Response = ").concat(type, ";");
            }
        })
            .join('\n\n') +
        '\n');
};
/**
 * Extract all types from #/components/schemas
 */
var generateSchemas = function (_a) {
    var _b, _c, _d;
    var spec = _a.spec;
    var schemaOutput = generateRequestBodiesDefinition((_b = spec.components) === null || _b === void 0 ? void 0 : _b.requestBodies) +
        generateResponsesDefinition((_c = spec.components) === null || _c === void 0 ? void 0 : _c.responses) +
        generateSchemasDefinition((_d = spec.components) === null || _d === void 0 ? void 0 : _d.schemas);
    return schemaOutput;
};
exports.generateSchemas = generateSchemas;
